#define MOSFET        5
#define BLOCK_SHOT    6
#define EDM_VOLT     A5
#define SENSOR_VOLT  A4
#define CNC_CONTROL  10

#define TOP_V       300
#define THRESHOLD_CHARGE    210
#define THRESHOLD_DISCHARGE 200
#define CURRENT_MAX          80

char text_buffer[100];

int edm_volt = 0;
int edm_volt_old = 0;
int sensor_volt = 0;
int shot_state = 0;

#define SHOT_PAUSE   10000000 
#define LOAD_DUTY    180


void establishContact() {
  while (Serial.available() <= 0) {
    Serial.print('X');   // send a capital X
    delay(300);
  }
}

// the setup function runs once when you press reset or power the board
void setup() {
  // initialize digital pin LED_BUILTIN as an output.
  pinMode(LED_BUILTIN, OUTPUT);
  pinMode(MOSFET, OUTPUT);
  pinMode(BLOCK_SHOT, OUTPUT);
  digitalWrite(BLOCK_SHOT, LOW);

  pinMode(CNC_CONTROL, OUTPUT);
  digitalWrite(CNC_CONTROL, LOW);

  shot_state = 0;
  
  Serial.begin(115200);
  while (!Serial) {
    ; // wait for serial port to connect. Needed for native USB port only
  }
  //establishContact();  // send a byte to establish contact until receiver responds
  Serial.println("Setup done.");
}

// the loop function runs over and over again forever
void loop() {
  if(shot_state == 0){ // Look for short circuit
    sensor_volt = analogRead(SENSOR_VOLT);
    analogWrite(BLOCK_SHOT, 10);
    if(sensor_volt > CURRENT_MAX){
      analogWrite(BLOCK_SHOT, 0);
      shot_state = -1;
      digitalWrite(CNC_CONTROL, LOW);
      Serial.println("SHORT DETECTED !!!");
    }
    else{
      Serial.println("...going to state 1");
      shot_state = 1;
      analogWrite(BLOCK_SHOT, 255);
    }
    //delay(1000);
  }

  if(shot_state == 1){// Loading capacitor
    edm_volt = analogRead(EDM_VOLT);
    sensor_volt = analogRead(SENSOR_VOLT);

    if(edm_volt < THRESHOLD_CHARGE){
      if(sensor_volt > CURRENT_MAX){
        analogWrite(BLOCK_SHOT, 0);
        shot_state = -1;
        digitalWrite(CNC_CONTROL, LOW);
        Serial.println("SHORT DETECTED !!!");        
      }
    }
    else{
      shot_state = 2;
      Serial.println("...going to state 2");
    }

    if(edm_volt < TOP_V){
      analogWrite(MOSFET, LOAD_DUTY);
    }
    else{
      analogWrite(MOSFET, 0);
      shot_state = 2;
    }
    
  }
  
  if(shot_state == 2 || shot_state == 3){
    edm_volt = analogRead(EDM_VOLT);
    if(edm_volt < TOP_V){
      analogWrite(MOSFET, LOAD_DUTY);
    }
    else{
      analogWrite(MOSFET, 0);
      if(shot_state == 2){
        shot_state = 3; 
        Serial.println("Fully loaded");
        digitalWrite(CNC_CONTROL, HIGH);
      }
    }

    if(edm_volt < THRESHOLD_DISCHARGE){
      analogWrite(BLOCK_SHOT, 0);
      shot_state = 0;
      digitalWrite(CNC_CONTROL, LOW);
      Serial.println("Regular shot, state = 0");
      delay(100);
    }
  }


  if (shot_state == -1){
    int inByte = Serial.read();
    sprintf(text_buffer, "EDM=%d, sensor=%d", edm_volt, sensor_volt);
    Serial.println(text_buffer);
    delay(1000);
    shot_state = 0;
  }

}
